from math import pi, sin, cos
import optparse
from scipy.signal import ZerosPolesGain
from scipy.signal import ellip

# Mark Tillotson, 2017-07-19
# Digital filter example
# Elliptical complex filter for quadrature generation of SSB

from utils import quantize, max_gain
from plotting import plot_response, plot_everything
from propeller import compile_propeller_filter


parser = optparse.OptionParser ()
parser.add_option ('--bits', action = 'store', type = int, default = '16')
parser.add_option ('--spin', action = 'store', default = None)
options, args = parser.parse_args ()

bits = options.bits
assert 4 <= bits <= 24

print ("Spin %s" % options.spin)

sysfreq = 87000
fs = 48000 # 0.25 * sysfreq
nyquist = 0.5 * fs
cut = 1200.0
stop_band_dB = 55
ripple_dB = 1.3


z, p, g = ellip (5, ripple_dB, stop_band_dB, cut/nyquist, btype = 'lowpass', analog = False, output = 'zpk')
resp = ZerosPolesGain (z, p, g)
zeros = resp.zeros
poles = resp.poles
gain = resp.gain

angle = (cut+250)/fs * 2*pi
a = cos(angle) + sin(angle) * 1j
zeros = [ quantize (zero * a, bits) for zero in zeros ]
poles = [ quantize (pole * a, bits) for pole in poles ]
resp = ZerosPolesGain (zeros, poles, gain)

resp = resp.to_tf ()
print ("Max gain = %s, gain_factor = %s" % (max_gain (resp), gain))
plot_everything (resp, fs, complex_filter = True)
plot_response (resp, fs, logarithmic = False)
plot_response (resp, fs, logarithmic = True)


print ("Spin %s" % options.spin)
if options.spin is not None:
    compile_propeller_filter (resp, options.spin, options.bits)

"""
////////////////////////////////////////////////////////////////////////////////////////////
//                                TERMS OF USE: MIT License
////////////////////////////////////////////////////////////////////////////////////////////
// Permission is hereby granted, free of charge, to any person obtaining a copy of this 
// software and associated documentation files (the "Software"), to deal in the Software 
// without restriction, including without limitation the rights to use, copy, modify, merge,
// publish, distribute, sublicense, and/or sell copies of the Software, and to permit 
// persons to whom the Software is furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all copies or
// substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
// INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
// PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE
// FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR 
// OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
// DEALINGS IN THE SOFTWARE.
////////////////////////////////////////////////////////////////////////////////////////////
"""
